package boot.common.respond;

import com.alibaba.fastjson.annotation.JSONField;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.fasterxml.jackson.annotation.JsonFormat;
import lombok.EqualsAndHashCode;

import java.io.Serializable;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;


@EqualsAndHashCode
public class ApiResult<T> implements Serializable {
    private static final long serialVersionUID = 8004487252556526569L;
//    @ApiModelProperty("响应码")
    private int status;
//    @ApiModelProperty("是否成功：成功true，失败false")
    private boolean success;
//    @ApiModelProperty("响应消息")
    private String msg;
//    @ApiModelProperty("响应数据")
    private T data;
//    @ApiModelProperty("响应时间")
    @JSONField(
            format = "yyyy-MM-dd HH:mm:ss"
    )
    @JsonFormat(
            pattern = "yyyy-MM-dd HH:mm:ss",
            timezone = "GMT+8"
    )
    private Date time;

    public ApiResult() {
        this.time = new Date();
    }

    public static ApiResult<Boolean> result(boolean flag) {
        return flag ? ok() : fail();
    }

    public static ApiResult<Boolean> result(ApiCode apiCode) {
        return result(apiCode, (Boolean) null);
    }

    public static <T> ApiResult<T> result(ApiCode apiCode, T data) {
        return result(apiCode, (String)null, data);
    }

    public static <T> ApiResult<T> result(ApiCode apiCode, String message, T data) {
        boolean success = false;
        if (apiCode.getCode() == ApiCode.SUCCESS.getCode()) {
            success = true;
        }

        if (StringUtils.isBlank(message)) {
            message = apiCode.getMessage();
        }

        return (ApiResult<T>) ApiResult.builder().status(apiCode.getCode()).msg(message).data(data).success(success).time(new Date()).build();
    }

    public static ApiResult<Boolean> ok() {
        return ok((Boolean)null);
    }

    public static <T> ApiResult<T> ok(T data) {
        return result(ApiCode.SUCCESS, data);
    }

    public static <T> ApiResult<T> ok(T data, String message) {
        return result(ApiCode.SUCCESS, message, data);
    }

    public static ApiResult<Map<String, Object>> okMap(String key, Object value) {
        Map<String, Object> map = new HashMap(1);
        map.put(key, value);
        return ok(map);
    }

    public static ApiResult<Boolean> fail(ApiCode apiCode) {
        return result(apiCode, (Boolean) null);
    }

    public static ApiResult<String> fail(String message) {
        return result(ApiCode.FAIL, message, (String) null);
    }

    public static <T> ApiResult<T> fail(ApiCode apiCode, T data) {
        if (ApiCode.SUCCESS == apiCode) {
            throw new RuntimeException("失败结果状态码不能为" + ApiCode.SUCCESS.getCode());
        } else {
            return result(apiCode, data);
        }
    }

    public static ApiResult<String> fail(Integer errorCode, String message) {
        return new ApiResult().setSuccess(false).setStatus(errorCode).setMsg(message);
    }

    public static ApiResult<Map<String, Object>> fail(String key, Object value) {
        Map<String, Object> map = new HashMap(1);
        map.put(key, value);
        return result(ApiCode.FAIL, map);
    }

    public static ApiResult<Boolean> fail() {
        return fail(ApiCode.FAIL);
    }

    public static <T> ApiResultBuilder<T> builder() {
        return new ApiResultBuilder<>();
    }

    public int getStatus() {
        return this.status;
    }

    public boolean isSuccess() {
        return this.success;
    }

    public String getMsg() {
        return this.msg;
    }

    public T getData() {
        return this.data;
    }

    public Date getTime() {
        return this.time;
    }

    public ApiResult<T> setStatus(final int status) {
        this.status = status;
        return this;
    }

    public ApiResult<T> setSuccess(final boolean success) {
        this.success = success;
        return this;
    }

    public ApiResult<T> setMsg(final String msg) {
        this.msg = msg;
        return this;
    }

    public ApiResult<T> setData(final T data) {
        this.data = data;
        return this;
    }

    @JsonFormat(
            pattern = "yyyy-MM-dd HH:mm:ss",
            timezone = "GMT+8"
    )
    public ApiResult<T> setTime(final Date time) {
        this.time = time;
        return this;
    }



    protected boolean canEqual(final Object other) {
        return other instanceof ApiResult;
    }


    public String toString() {
        return "ApiResult(status=" + this.getStatus() + ", success=" + this.isSuccess() + ", msg=" + this.getMsg() + ", data=" + this.getData() + ", time=" + this.getTime() + ")";
    }

    public ApiResult(final int status, final boolean success, final String msg, final T data, final Date time) {
        this.status = status;
        this.success = success;
        this.msg = msg;
        this.data = data;
        this.time = time;
    }

    public static class ApiResultBuilder<T> {
        private int status;
        private boolean success;
        private String msg;
        private T data;
        private Date time;

        ApiResultBuilder() {
        }

        public ApiResultBuilder<T> status(final int status) {
            this.status = status;
            return this;
        }

        public ApiResultBuilder<T> success(final boolean success) {
            this.success = success;
            return this;
        }

        public ApiResultBuilder<T> msg(final String msg) {
            this.msg = msg;
            return this;
        }

        public ApiResultBuilder<T> data(final T data) {
            this.data = data;
            return this;
        }

        @JsonFormat(
                pattern = "yyyy-MM-dd HH:mm:ss",
                timezone = "GMT+8"
        )
        public ApiResultBuilder<T> time(final Date time) {
            this.time = time;
            return this;
        }

        public ApiResult<T> build() {
            return new ApiResult(this.status, this.success, this.msg, this.data, this.time);
        }

        public String toString() {
            return "ApiResult.ApiResultBuilder(status=" + this.status + ", success=" + this.success + ", msg=" + this.msg + ", data=" + this.data + ", time=" + this.time + ")";
        }
    }
}
