package boot.modules.user.service.impl;

import boot.common.bean.OnlineUser;
import boot.common.constant.SchoolConstant;
import boot.common.respond.EException;
import boot.common.utils.EncryptUtils;
import boot.common.utils.IpUtil;
import boot.common.utils.RedisUtils;
import boot.common.utils.StringUtils;
import boot.modules.user.param.RegParam;
import boot.modules.user.pojo.User;
import boot.modules.user.service.UserService;
import cn.hutool.core.net.Ipv4Util;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.SecureUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@Transactional(rollbackFor = RuntimeException.class)
public class AuthService {

    private final UserService userService;
    private final RedisUtils redisUtils;
    private static Integer expiredTimeIn;

    @Value("${school.security.token-expired-in}")
    public void setExpiredTimeIn(Integer expiredTimeIn) {
        AuthService.expiredTimeIn = expiredTimeIn;
    }

    /**
     * 注册
     * @param param RegDTO
     */
    public void register(RegParam param){

        String account = param.getAccount();
        String ip = IpUtil.getRequestIp();
        User user = User.builder()
                .username(account)
                .nickname(param.getNickname())
                .password(SecureUtil.md5(param.getPassword()))
                .phone(account)
                .avatar(null)
                .addIp(ip)
                .lastIp(ip)
                .build();

        userService.save(user);

    }


    /**
     * 保存在线用户信息
     * @param user /
     * @param token /
     * @param request /
     */
    public void save(User user, String token, HttpServletRequest request){
        String job = "开发工程师";
        String ip = IpUtil.getRequestIp();
        String browser = StringUtils.getBrowser(request);
        String address = StringUtils.getCityInfo(ip);
        OnlineUser onlineUser = null;
        try {
            onlineUser = new OnlineUser(user.getUsername(), user.getNickname(), job, browser ,
                    ip, address, EncryptUtils.desEncrypt(token), new Date());
        } catch (Exception e) {
            e.printStackTrace();
        }
        redisUtils.set(SchoolConstant.SCHOOL_LOGIN_USER + onlineUser.getUserName() + ":" + token, onlineUser, AuthService.expiredTimeIn);
    }

    /**
     * 检测用户是否在之前已经登录，已经登录踢下线
     *
     * @param userName 用户名
     */
    public void checkLoginOnUser(String userName, String igoreToken) {
        List<OnlineUser> onlineUsers = this.getAll(userName);
        if (onlineUsers == null || onlineUsers.isEmpty()) {
            return;
        }
        for (OnlineUser onlineUser : onlineUsers) {
            try {
                String token = EncryptUtils.desDecrypt(onlineUser.getKey());
                if (StrUtil.isNotBlank(igoreToken) && !igoreToken.equals(token)) {
                    this.kickOut(userName, onlineUser.getKey());
                } else if (StrUtil.isBlank(igoreToken)) {
                    this.kickOut(userName, onlineUser.getKey());
                }
            } catch (Exception e) {
                log.error("checkUser is error", e);
            }
        }
    }

    /**
     * 踢出用户
     *
     * @param key /
     */
    public void kickOut(String userName, String key) throws Exception {
        key = SchoolConstant.SCHOOL_LOGIN_USER + userName + ":" + EncryptUtils.desDecrypt(key);
        redisUtils.del(key);
    }

    /**
     * 退出登录
     * @param token /
     */
    public void logout(String userName,String token) {
        String key = SchoolConstant.SCHOOL_LOGIN_USER+ userName + ":" + token;
        redisUtils.del(key);
    }

    /**
     * 查询全部数据，不分页
     *
     * @param uName /
     * @return /
     */
    private List<OnlineUser> getAll(String uName) {
        List<String> keys = null;
        keys = redisUtils.scan(SchoolConstant.SCHOOL_LOGIN_USER + uName + ":" + "*");

        Collections.reverse(keys);
        List<OnlineUser> onlineUsers = new ArrayList<>();
        for (String key : keys) {
            OnlineUser onlineUser = (OnlineUser) redisUtils.get(key);
            onlineUsers.add(onlineUser);
        }
        onlineUsers.sort((o1, o2) -> o2.getLoginTime().compareTo(o1.getLoginTime()));
        return onlineUsers;
    }
}

